<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/2/16
 * Time: 10:57 PM
 */

namespace qfq;

use qfq;

//use qfq\CodeException;
//use qfq\UserFormException;

require_once(__DIR__ . '/../../qfq/Constants.php');
//require_once(__DIR__ . '/../exceptions/UserFormException.php');

/**
 * Class Sanitize
 * @package qfq
 */
class Sanitize {


    private function __construct() {
        // Class should never be instantiated
    }

    /**
     * Check $value against given checkType/pattern. If check succeed, returns values.
     *   If check fails, depending on $mode, throws an UserException or return an empty string.
     *
     * @param string $value value to check
     * @param string $sanatizeClass SANITIZE_ALLOW_*
     * @param string $patternOrRange Pattern as regexp or MIN|MAX values
     * @param string $mode SANATIZE_EXCEPTION | SANATIZE_EMPTY_STRING
     * @return string
     * @throws UserFormException
     * @throws \qfq\CodeException
     */
    public static function sanitize($value, $sanatizeClass = SANITIZE_DEFAULT, $patternOrRange = '', $mode = SANATIZE_EMPTY_STRING) {
        $pattern = '';
        $minMax = array();
        $valueCompare = '';
        $errorCode = 0;
        $errorText = '';

        // Prepare MIN|MAX
        switch ($sanatizeClass) {
            case SANITIZE_ALLOW_MIN_MAX:
                $minMax = explode('|', $patternOrRange);
                $valueCompare = $value;
                break;

            case SANITIZE_ALLOW_MIN_MAX_DATE:
                $minMax = explode('|', $patternOrRange);

                //TODO: hier sollten die Exceptions abgefangen werden um zwei unterschiedliche Fehlermeldungen ausgeben zu koenenn:
                // a) der Value verletzt die Datumsgrenzen
                // b) die Definition der Grenzen ist buggy
//                try {
                $valueCompare = Support::dateTimeGermanToInternational($value);
//                } catch (UserFormException $e) {
//                    throw new UserFormException("Date or time not recognized '" . $value . "' - " . $e->formatMessage(), ERROR_SANATIZE_INVALID_VALUE);
//                }

//                try {
                $minMax[0] = Support::dateTimeGermanToInternational($minMax[0]);
                $minMax[1] = Support::dateTimeGermanToInternational($minMax[1]);
//                } catch  (UserFormException $e) {
//                    throw new UserFormException("Date or time of min|max definition not recognized '" . $patternOrRange . "' - " . $e->formatMessage(), ERROR_SANATIZE_INVALID_VALUE);
//                }

                break;

            default:
                break;
        }

        // Prepare Check
        switch ($sanatizeClass) {
            case SANITIZE_ALLOW_MIN_MAX:
            case SANITIZE_ALLOW_MIN_MAX_DATE:

                if ($minMax[0] === '' || $minMax[1] === '') {
                    throw new UserFormException('Missing definition of value for min or max.', ERROR_MISSING_MIN_MAX);
                }
            $errorText = "Value '$value' is out of range of '$patternOrRange'.";

                if ($minMax[0] <= $valueCompare && $valueCompare <= $minMax[1])
                    return $value;

                $errorCode = ERROR_MIN_MAX_VIOLATION;
                break;

            case SANITIZE_ALLOW_PATTERN:
                $pattern = $patternOrRange;
                break;

            case SANITIZE_ALLOW_DIGIT:
            case SANITIZE_ALLOW_NUMERICAL:
            case SANITIZE_ALLOW_EMAIL:
            case SANITIZE_ALLOW_ALNUMX:
            case SANITIZE_ALLOW_ALLBUT:
                $arr = self::inputCheckPatternArray();
                $pattern = $arr[$sanatizeClass];
                break;

            case SANITIZE_ALLOW_ALL: // no checktype specified.
                return $value;

            default:
                throw new CodeException("Unknown checkType: " . $sanatizeClass, ERROR_UNKNOWN_CHECKTYPE);
        }

        // No error until here: do a final check
        if ($errorCode == 0) {
            if (preg_match("/$pattern/", $value) === 1)
                return $value;
            else
                $errorCode = ERROR_PATTERN_VIOLATION;
        }

        if ($mode === SANATIZE_EXCEPTION) {
            if ($errorText === '')
                $errorText = "Value '$value' violates checkrule " . $sanatizeClass . " with pattern '$pattern'.";
            throw new UserFormException($errorText, $errorCode);
        }

        // check failed: return empty string
        return '';
    }

    /**
     * @return array
     */
    public static function inputCheckPatternArray() {
        //EMail Regex: http://www.regular-expressions.info/email.html
        return [
            SANITIZE_ALLOW_ALNUMX => '^[@\-_\.,;: \/\(\)a-zA-Z0-9ÀÈÌÒÙàèìòùÁÉÍÓÚÝáéíóúýÂÊÎÔÛâêîôûÃÑÕãñõÄËÏÖÜŸäëïöüÿ]*$', // ':alnum:' does not work here in FF
            SANITIZE_ALLOW_DIGIT => '^[\d]*$',
            SANITIZE_ALLOW_NUMERICAL => '^[\d.+-]*$',
            SANITIZE_ALLOW_EMAIL => '^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$',
            SANITIZE_ALLOW_MIN_MAX => '',
            SANITIZE_ALLOW_MIN_MAX_DATE => '',
            SANITIZE_ALLOW_PATTERN => '',
            SANITIZE_ALLOW_ALLBUT => '^[^\[\]{}%&\\\\#]*$',
            SANITIZE_ALLOW_ALL => '.*'
        ];
    }

    /**
     * Sanatizes a filename. Copied from http://www.phpit.net/code/filename-safe/
     *
     * @param $filename
     * @return mixed
     */
    public static function safeFilename($filename) {
        $search = array(
            // Definition of German Umlauts START
            '/ß/',
            '/ä/', '/Ä/',
            '/ö/', '/Ö/',
            '/ü/', '/Ü/',
            // Definition of German Umlauts ENDE
            '([^[:alnum:]._])' // Disallow: Not alphanumeric, dot or underscore
        );

        $replace = array(
            'ss',
            'ae', 'Ae',
            'oe', 'Oe',
            'ue', 'Ue',
            '_'
        );

        return preg_replace($search, $replace, $filename);
    } // safeFilename()

    /**
     * Take the given $item (or iterates over all elements of the given array) and normalize the content.
     * Only strings will be normalized. Sub arrays will be recursived normalized. Numeric content is skipped.
     * Throws an exception for unknown content.
     *
     * It's important to normalize the user input: e.g. someone is searching for a record and input the search string
     * with composed characters (happens e.g. on Apple Mac / Safari without special user invention).
     *
     * @param array|string $item
     * @return array|string
     * @throws CodeException
     */
    public static function normalize($item) {

        if (is_array($item)) {
            foreach ($item as $key => $value) {
                $value = self::normalize($value);
                $item[$key] = $value;
            }
        } else {
            if (is_string($item)) {
                $item = \normalizer::normalize($item, \Normalizer::FORM_C);
            } elseif (!is_numeric($item)) {
                throw new qfq\CodeException ("Expect type 'string / numeric / array' - but there is something else.", ERROR_UNEXPECTED_TYPE);
            }
        }
        return $item;
    }
}